/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Common Log Format (CLF) CDR records.
 * by Brian K. West <brian@bkw.org>
 *
 *
 * Copyright (C) 2004, Digium
 *
 * Based on work by Mark Spencer <markster@linux-support.net>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License.
 *
 */

#include <sys/types.h>
#include <asterisk/channel.h>
#include <asterisk/cdr.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include "../asterisk.h"
#include "../astconf.h"

#define CLF_LOG_DIR "/cdr-clf"
#define CLF_MASTER  "/access_log"

#define DATE_FORMAT "[%d/%b/%Y:%H:%M:%S %z]"

#include <stdio.h>
#include <string.h>

#include <stdlib.h>
#include <unistd.h>
#include <time.h>

static char *desc = "Common Log Format (CLF) CDR Backend";

static char *name = "clf";

static FILE *mf = NULL;

static int build_clf_record(char *buf, int len, struct ast_cdr *cdr) {
	char tmp[80];
	struct tm tm;
	time_t t;
	t = cdr->end.tv_sec;
        
	localtime_r(&t,&tm);
	strftime(tmp, sizeof(tmp), DATE_FORMAT, &tm);

	snprintf(buf, len, "%s - - %s \"GET /%s/%s\" 200 %d\n", cdr->src, tmp, cdr->dcontext, cdr->dst, cdr->billsec * 1024);

	/* If we hit the end of our buffer, log an error */
	if (strlen(buf) < len - 5) {
		return 0;
	}
	return -1;
}

static int writefile(char *s, char *acc) {
	char tmp[256];
	FILE *f;
	if (strchr(acc, '/') || (acc[0] == '.')) {
		ast_log(LOG_WARNING, "Account code '%s' insecure for writing file\n", acc);
		return -1;
	}
	snprintf(tmp, sizeof(tmp), "%s/%s/%s_log", (char *)ast_config_AST_LOG_DIR,CLF_LOG_DIR, acc);
	f = fopen(tmp, "a");
	if (!f)
		return -1;
	fputs(s, f);
	fclose(f);
	return 0;
}


static int clf_log(struct ast_cdr *cdr) {
	/* Make sure we have a big enough buf */
	char buf[1024];
	char clfmaster[AST_CONFIG_MAX_PATH];
	snprintf((char *)clfmaster,sizeof(clfmaster)-1,"%s/%s/%s",(char *)ast_config_AST_LOG_DIR,CLF_LOG_DIR,CLF_MASTER);
#if 0
	printf("[CDR] %s ('%s' -> '%s') Dur: %ds Bill: %ds Disp: %s Flags: %s Account: [%s]\n", cdr->channel, cdr->src, cdr->dst, cdr->duration, cdr->billsec, ast_cdr_disp2str(cdr->disposition), ast_cdr_flags2str(cdr->amaflags), cdr->accountcode);
#endif
	if (build_clf_record(buf, sizeof(buf), cdr)) {
		ast_log(LOG_WARNING, "Unable to create CLF record in %d bytes.  CDR not recorded!\n", sizeof(buf));
	} else {
		/* because of the absolutely unconditional need for the
		   highest reliability possible in writing billing records,
		   we open write and close the log file each time */
		mf = fopen(clfmaster, "a");
		if (!mf) {
			ast_log(LOG_ERROR, "Unable to re-open master file %s\n", clfmaster);
		}
		if (mf) {
			fputs(buf, mf);
			fflush(mf); /* be particularly anal here */
			fclose(mf);
			mf = NULL;
		}
		if (strlen(cdr->accountcode)) {
			if (writefile(buf, cdr->accountcode))
				ast_log(LOG_WARNING, "Unable to write CLF record to account file '%s'\n", cdr->accountcode);
		}
	}
	return 0;
}

char *description(void) {
	return desc;
}

int unload_module(void) {
	if (mf)
		fclose(mf);
	ast_cdr_unregister(name);
	return 0;
}

int load_module(void) {
	int res;

	res = ast_cdr_register(name, desc, clf_log);
	if (res) {
		ast_log(LOG_ERROR, "Unable to register CLF CDR handling\n");
		if (mf)
			fclose(mf);
	}
	return res;
}

int reload(void) {
	return 0;
}

int usecount(void) {
	return 0;
}

char *key() {
	return ASTERISK_GPL_KEY;
}
